@@ -24,6 +24,8 @@ class InvalidTemplateError(Error):
     pass
 class UnknownWebhookError(Error):
     pass
+class UnknownInboundDomainError(Error):
+    pass
 
 ROOT = 'https://mandrillapp.com/api/1.0/'
 ERROR_MAP = {
@@ -35,7 +37,8 @@ ERROR_MAP = {
     'Unknown_Sender': UnknownSenderError,
     'Unknown_Url': UnknownUrlError,
     'Invalid_Template': InvalidTemplateError,
-    'Unknown_Webhook': UnknownWebhookError
+    'Unknown_Webhook': UnknownWebhookError,
+    'Unknown_InboundDomain': UnknownInboundDomainError
 }
 
 logger = logging.getLogger('mandrill')
@@ -73,8 +76,10 @@ class Mandrill(object):
         self.templates = Templates(self)
         self.users = Users(self)
         self.rejects = Rejects(self)
+        self.inbound = Inbound(self)
         self.tags = Tags(self)
         self.messages = Messages(self)
+        self.internal = Internal(self)
         self.urls = Urls(self)
         self.webhooks = Webhooks(self)
         self.senders = Senders(self)
@@ -86,7 +91,7 @@ class Mandrill(object):
         params = json.dumps(params)
         self.log('POST to %s%s.json: %s' % (ROOT, url, params))
         start = time.time()
-        r = self.session.post('%s%s.json' % (ROOT, url), data=params, headers={'content-type': 'application/json', 'user-agent': 'Mandrill-Python/1.0.6'})
+        r = self.session.post('%s%s.json' % (ROOT, url), data=params, headers={'content-type': 'application/json', 'user-agent': 'Mandrill-Python/1.0.13'})
         try:
             remote_addr = r.raw._original_response.fp._sock.getpeername() # grab the remote_addr before grabbing the text since the socket will go away
         except:
@@ -147,9 +152,10 @@ class Templates(object):
 
         Returns:
            struct.  the information saved about the new template::
-               name (string): the name of the template - draft version
+               slug (string): the immutable unique code name of the template
+               name (string): the name of the template
                code (string): the full HTML code of the template, with mc:edit attributes marking the editable elements - draft version
-               publish_name (string): the name that the template was published as, if it has been published
+               publish_name (string): the same as the template name - kept as a separate field for backwards compatibility
                publish_code (string): the full HTML code of the template, with mc:edit attributes marking the editable elements that are available as published, if it has been published
                published_at (string): the date and time the template was last published as a UTC string in YYYY-MM-DD HH:MM:SS format, or null if it has not been published
                created_at (string): the date and time the template was first created as a UTC string in YYYY-MM-DD HH:MM:SS format
@@ -167,13 +173,14 @@ class Templates(object):
         """Get the information for an existing template
 
         Args:
-           name (string): the name of an existing template
+           name (string): the immutable name of an existing template
 
         Returns:
            struct.  the requested template information::
-               name (string): the name of the template - draft version
+               slug (string): the immutable unique code name of the template
+               name (string): the name of the template
                code (string): the full HTML code of the template, with mc:edit attributes marking the editable elements - draft version
-               publish_name (string): the name that the template was published as, if it has been published
+               publish_name (string): the same as the template name - kept as a separate field for backwards compatibility
                publish_code (string): the full HTML code of the template, with mc:edit attributes marking the editable elements that are available as published, if it has been published
                published_at (string): the date and time the template was last published as a UTC string in YYYY-MM-DD HH:MM:SS format, or null if it has not been published
                created_at (string): the date and time the template was first created as a UTC string in YYYY-MM-DD HH:MM:SS format
@@ -191,15 +198,16 @@ class Templates(object):
         """Update the code for an existing template
 
         Args:
-           name (string): the name of an existing template
+           name (string): the immutable name of an existing template
            code (string): the new code for the template
            publish (boolean): set to false to update the draft version of the template without publishing
 
         Returns:
            struct.  the template that was updated::
-               name (string): the name of the template - draft version
+               slug (string): the immutable unique code name of the template
+               name (string): the name of the template
                code (string): the full HTML code of the template, with mc:edit attributes marking the editable elements - draft version
-               publish_name (string): the name that the template was published as, if it has been published
+               publish_name (string): the same as the template name - kept as a separate field for backwards compatibility
                publish_code (string): the full HTML code of the template, with mc:edit attributes marking the editable elements that are available as published, if it has been published
                published_at (string): the date and time the template was last published as a UTC string in YYYY-MM-DD HH:MM:SS format, or null if it has not been published
                created_at (string): the date and time the template was first created as a UTC string in YYYY-MM-DD HH:MM:SS format
@@ -217,13 +225,14 @@ class Templates(object):
         """Publish the content for the template. Any new messages sent using this template will start using the content that was previously in draft.
 
         Args:
-           name (string): the name of an existing template
+           name (string): the immutable name of an existing template
 
         Returns:
            struct.  the template that was published::
-               name (string): the name of the template - draft version
+               slug (string): the immutable unique code name of the template
+               name (string): the name of the template
                code (string): the full HTML code of the template, with mc:edit attributes marking the editable elements - draft version
-               publish_name (string): the name that the template was published as, if it has been published
+               publish_name (string): the same as the template name - kept as a separate field for backwards compatibility
                publish_code (string): the full HTML code of the template, with mc:edit attributes marking the editable elements that are available as published, if it has been published
                published_at (string): the date and time the template was last published as a UTC string in YYYY-MM-DD HH:MM:SS format, or null if it has not been published
                created_at (string): the date and time the template was first created as a UTC string in YYYY-MM-DD HH:MM:SS format
@@ -241,13 +250,14 @@ class Templates(object):
         """Delete a template
 
         Args:
-           name (string): the name of an existing template
+           name (string): the immutable name of an existing template
 
         Returns:
            struct.  the template that was deleted::
-               name (string): the name of the template - draft version
+               slug (string): the immutable unique code name of the template
+               name (string): the name of the template
                code (string): the full HTML code of the template, with mc:edit attributes marking the editable elements - draft version
-               publish_name (string): the name that the template was published as, if it has been published
+               publish_name (string): the same as the template name - kept as a separate field for backwards compatibility
                publish_code (string): the full HTML code of the template, with mc:edit attributes marking the editable elements that are available as published, if it has been published
                published_at (string): the date and time the template was last published as a UTC string in YYYY-MM-DD HH:MM:SS format, or null if it has not been published
                created_at (string): the date and time the template was first created as a UTC string in YYYY-MM-DD HH:MM:SS format
@@ -267,9 +277,10 @@ class Templates(object):
         Returns:
            array.  an array of structs with information about each template::
                [] (struct): the information on each template in the account::
-                   [].name (string): the name of the template - draft version
+                   [].slug (string): the immutable unique code name of the template
+                   [].name (string): the name of the template
                    [].code (string): the full HTML code of the template, with mc:edit attributes marking the editable elements - draft version
-                   [].publish_name (string): the name that the template was published as, if it has been published
+                   [].publish_name (string): the same as the template name - kept as a separate field for backwards compatibility
                    [].publish_code (string): the full HTML code of the template, with mc:edit attributes marking the editable elements that are available as published, if it has been published
                    [].published_at (string): the date and time the template was last published as a UTC string in YYYY-MM-DD HH:MM:SS format, or null if it has not been published
                    [].created_at (string): the date and time the template was first created as a UTC string in YYYY-MM-DD HH:MM:SS format
@@ -316,7 +327,7 @@ class Templates(object):
         """Inject content and optionally merge fields into a template, returning the HTML that results
 
         Args:
-           template_name (string): the name of a template that exists in the user's account
+           template_name (string): the immutable name of a template that exists in the user's account
            template_content (array): an array of template content to render.  Each item in the array should be a struct with two keys - name: the name of the content block to set the content for, and content: the actual content to put into the block::
                template_content[] (struct): the injection of a single piece of content into a single editable region::
                    template_content[].name (string): the name of the mc:edit editable region to inject into
@@ -494,12 +505,15 @@ class Rejects(object):
     def __init__(self, master):
         self.master = master
 
-    def list(self, email=None):
+    def list(self, email=None, include_expired=False):
         """Retrieves your email rejection blacklist. You can provide an email
-address to limit the results. Returns up to 1000 results.
+address to limit the results. Returns up to 1000 results. By default,
+entries that have expired are excluded from the results; set
+include_expired to true to include them.
 
         Args:
            email (string): an optional email address to search by
+           include_expired (boolean): whether to include rejections that have already expired.
 
         Returns:
            array.  Up to 1000 rejection entries::
@@ -516,7 +530,7 @@ address to limit the results. Returns up to 1000 results.
            InvalidKeyError: The provided API key is not a valid Mandrill API key
            Error: A general Mandrill error has occurred
         """
-        _params = {'email': email}
+        _params = {'email': email, 'include_expired': include_expired}
         return self.master.call('rejects/list', _params)
 
     def delete(self, email):
@@ -541,6 +555,69 @@ has an affect on your reputation.
         return self.master.call('rejects/delete', _params)
 
 
+class Inbound(object):
+    def __init__(self, master):
+        self.master = master
+
+    def domains(self, ):
+        """List the domains that have been configured for inbound delivery
+
+        Returns:
+           array.  the inbound domains associated with the account::
+               [] (array): the individual domain info
+
+        Raises:
+           InvalidKeyError: The provided API key is not a valid Mandrill API key
+           Error: A general Mandrill error has occurred
+        """
+        _params = {}
+        return self.master.call('inbound/domains', _params)
+
+    def routes(self, domain):
+        """List the mailbox routes defined for an inbound domain
+
+        Args:
+           domain (string): the domain to check
+
+        Returns:
+           array.  the routes associated with the domain::
+               [] (struct): the individual mailbox route::
+                   [].pattern (string): the search pattern that the mailbox name should match
+                   [].url (string): the webhook URL where inbound messages will be published
+
+
+        Raises:
+           InvalidKeyError: The provided API key is not a valid Mandrill API key
+           UnknownInboundDomainError: The requested inbound domain does not exist
+           Error: A general Mandrill error has occurred
+        """
+        _params = {'domain': domain}
+        return self.master.call('inbound/routes', _params)
+
+    def send_raw(self, raw_message, to=None):
+        """Take a raw MIME document destined for a domain with inbound domains set up, and send it to the inbound hook exactly as if it had been sent over SMTP
+$sparam string $to[] the email address of the recipient @validate trim
+
+        Args:
+           raw_message (string): the full MIME document of an email message
+           to (array|null): optionally define the recipients to receive the message - otherwise we'll use the To, Cc, and Bcc headers provided in the document
+
+        Returns:
+           array.  an array of the information for each recipient in the message (usually one) that matched an inbound route::
+               [] (struct): the individual recipient information::
+                   [].email (struct): the email address of the matching recipient
+                   [].pattern (struct): the mailbox route pattern that the recipient matched
+                   [].url (struct): the webhook URL that the message was posted to
+
+
+        Raises:
+           InvalidKeyError: The provided API key is not a valid Mandrill API key
+           Error: A general Mandrill error has occurred
+        """
+        _params = {'raw_message': raw_message, 'to': to}
+        return self.master.call('inbound/send-raw', _params)
+
+
 class Tags(object):
     def __init__(self, master):
         self.master = master
@@ -569,6 +646,34 @@ class Tags(object):
         _params = {}
         return self.master.call('tags/list', _params)
 
+    def delete(self, tag):
+        """Deletes a tag permanently. Deleting a tag removes the tag from any messages
+that have been sent, and also deletes the tag's stats. There is no way to
+undo this operation, so use it carefully.
+
+        Args:
+           tag (string): a tag name
+
+        Returns:
+           struct.  the tag that was deleted::
+               tag (string): the actual tag as a string
+               sent (integer): the total number of messages sent with this tag
+               hard_bounces (integer): the total number of hard bounces by messages with this tag
+               soft_bounces (integer): the total number of soft bounces by messages with this tag
+               rejects (integer): the total number of rejected messages with this tag
+               complaints (integer): the total number of spam complaints received for messages with this tag
+               unsubs (integer): the total number of unsubscribe requests received for messages with this tag
+               opens (integer): the total number of times messages with this tag have been opened
+               clicks (integer): the total number of times tracked URLs in messages with this tag have been clicked
+
+        Raises:
+           InvalidTagNameError: The requested tag does not exist or contains invalid characters
+           InvalidKeyError: The provided API key is not a valid Mandrill API key
+           Error: A general Mandrill error has occurred
+        """
+        _params = {'tag': tag}
+        return self.master.call('tags/delete', _params)
+
     def info(self, tag):
         """Return more detailed information about a single tag, including aggregates of recent stats
 
@@ -716,7 +821,7 @@ class Messages(object):
     def __init__(self, master):
         self.master = master
 
-    def send(self, message):
+    def send(self, message, async=False):
         """Send a new transactional message through Mandrill
 
         Args:
@@ -763,6 +868,12 @@ class Messages(object):
                message.google_analytics_domains (array): an array of strings indicating for which any matching URLs will automatically have Google Analytics parameters appended to their query string automatically.
                message.google_analytics_campaign (array|string): optional string indicating the value to set for the utm_campaign tracking parameter. If this isn't provided the email's from address will be used instead.
                message.metadata (array): metadata an associative array of user metadata. Mandrill will store this metadata and make it available for retrieval. In addition, you can select up to 10 metadata fields to index and make searchable using the Mandrill search api.
+               message.recipient_metadata (array): Per-recipient metadata that will override the global values specified in the metadata parameter.::
+                   message.recipient_metadata[] (struct): metadata for a single recipient::
+                       message.recipient_metadata[].rcpt (string): the email address of the recipient that the metadata is associated with
+                       message.recipient_metadata[].values (array): an associated array containing the recipient's unique metadata. If a key exists in both the per-recipient metadata and the global metadata, the per-recipient metadata will be used.
+
+
                message.attachments (array): an array of supported attachments to add to the message::
                    message.attachments[] (struct): a single supported attachment::
                        message.attachments[].type (string): the MIME type of the attachment - allowed types are text/*, image/*, and application/pdf
@@ -770,6 +881,7 @@ class Messages(object):
                        message.attachments[].content (string): the content of the attachment as a base64-encoded string
 
 
+           async (boolean): enable a background sending mode that is optimized for bulk sending. In async mode, messages/send will immediately return a status of "queued" for every recipient. To handle rejections when sending in async mode, set up a webhook for the 'reject' event. Defaults to false for messages with fewer than 100 recipients; messages with more than 100 recipients are always sent asynchronously, regardless of the value of async.
 
         Returns:
            array.  of structs for each recipient containing the key "email" with the email address and "status" as either "sent", "queued", or "rejected"::
@@ -782,10 +894,10 @@ class Messages(object):
            InvalidKeyError: The provided API key is not a valid Mandrill API key
            Error: A general Mandrill error has occurred
         """
-        _params = {'message': message}
+        _params = {'message': message, 'async': async}
         return self.master.call('messages/send', _params)
 
-    def send_template(self, template_name, template_content, message):
+    def send_template(self, template_name, template_content, message, async=False):
         """Send a new transactional message through Mandrill using a template
 
         Args:
@@ -836,6 +948,12 @@ class Messages(object):
                message.google_analytics_domains (array): an array of strings indicating for which any matching URLs will automatically have Google Analytics parameters appended to their query string automatically.
                message.google_analytics_campaign (array|string): optional string indicating the value to set for the utm_campaign tracking parameter. If this isn't provided the email's from address will be used instead.
                message.metadata (array): metadata an associative array of user metadata. Mandrill will store this metadata and make it available for retrieval. In addition, you can select up to 10 metadata fields to index and make searchable using the Mandrill search api.
+               message.recipient_metadata (array): Per-recipient metadata that will override the global values specified in the metadata parameter.::
+                   message.recipient_metadata[] (struct): metadata for a single recipient::
+                       message.recipient_metadata[].rcpt (string): the email address of the recipient that the metadata is associated with
+                       message.recipient_metadata[].values (array): an associated array containing the recipient's unique metadata. If a key exists in both the per-recipient metadata and the global metadata, the per-recipient metadata will be used.
+
+
                message.attachments (array): an array of supported attachments to add to the message::
                    message.attachments[] (struct): a single supported attachment::
                        message.attachments[].type (string): the MIME type of the attachment - allowed types are text/*, image/*, and application/pdf
@@ -843,6 +961,7 @@ class Messages(object):
                        message.attachments[].content (string): the content of the attachment as a base64-encoded string
 
 
+           async (boolean): enable a background sending mode that is optimized for bulk sending. In async mode, messages/sendTemplate will immediately return a status of "queued" for every recipient. To handle rejections when sending in async mode, set up a webhook for the 'reject' event. Defaults to false for messages with fewer than 100 recipients; messages with more than 100 recipients are always sent asynchronously, regardless of the value of async.
 
         Returns:
            array.  of structs for each recipient containing the key "email" with the email address and "status" as either "sent", "queued", or "rejected"::
@@ -856,7 +975,7 @@ class Messages(object):
            InvalidKeyError: The provided API key is not a valid Mandrill API key
            Error: A general Mandrill error has occurred
         """
-        _params = {'template_name': template_name, 'template_content': template_content, 'message': message}
+        _params = {'template_name': template_name, 'template_content': template_content, 'message': message, 'async': async}
         return self.master.call('messages/send-template', _params)
 
     def search(self, query='*', date_from=None, date_to=None, tags=None, senders=None, limit=100):
@@ -874,6 +993,7 @@ class Messages(object):
            array.  of structs for each matching message::
                [] (struct): the information for a single matching message::
                    [].ts (integer): the Unix timestamp from when this message was sent
+                   []._id (string): the message's unique id
                    [].sender (string): the email address of the sender
                    [].subject (string): the message's subject link
                    [].email (string): the recipient email address
@@ -883,6 +1003,7 @@ class Messages(object):
                    [].opens (integer): how many times has this message been opened
                    [].clicks (integer): how many times has a link been clicked in this message
                    [].state (string): sending status of this message: sent, bounced, rejected
+                   [].metadata (struct): any custom metadata provided when the message was sent
 
 
         Raises:
@@ -928,7 +1049,7 @@ class Messages(object):
         _params = {'raw_message': raw_message}
         return self.master.call('messages/parse', _params)
 
-    def send_raw(self, raw_message, from_email=None, from_name=None, to=None):
+    def send_raw(self, raw_message, from_email=None, from_name=None, to=None, async=False):
         """Take a raw MIME document for a message, and send it exactly as if it were sent over the SMTP protocol
 
         Args:
@@ -937,6 +1058,7 @@ class Messages(object):
            from_name (string|null): optionally define the sender alias
            to (array|null): optionally define the recipients to receive the message - otherwise we'll use the To, Cc, and Bcc headers provided in the document::
                to[] (string): the email address of the recipint
+           async (boolean): enable a background sending mode that is optimized for bulk sending. In async mode, messages/sendRaw will immediately return a status of "queued" for every recipient. To handle rejections when sending in async mode, set up a webhook for the 'reject' event. Defaults to false for messages with fewer than 100 recipients; messages with more than 100 recipients are always sent asynchronously, regardless of the value of async.
 
         Returns:
            array.  of structs for each recipient containing the key "email" with the email address and "status" as either "sent", "queued", or "rejected"::
@@ -949,10 +1071,15 @@ class Messages(object):
            InvalidKeyError: The provided API key is not a valid Mandrill API key
            Error: A general Mandrill error has occurred
         """
-        _params = {'raw_message': raw_message, 'from_email': from_email, 'from_name': from_name, 'to': to}
+        _params = {'raw_message': raw_message, 'from_email': from_email, 'from_name': from_name, 'to': to, 'async': async}
         return self.master.call('messages/send-raw', _params)
 
 
+class Internal(object):
+    def __init__(self, master):
+        self.master = master
+
+
 class Urls(object):
     def __init__(self, master):
         self.master = master
@@ -966,7 +1093,7 @@ class Urls(object):
                    [].url (string): the URL to be tracked
                    [].sent (integer): the number of emails that contained the URL
                    [].clicks (integer): the number of times the URL has been clicked from a tracked email
-                   [].unique_clicks (integer): the number of unique emails that have benerated clicks for this URL
+                   [].unique_clicks (integer): the number of unique emails that have generated clicks for this URL
 
 
         Raises:
@@ -988,7 +1115,7 @@ class Urls(object):
                    [].url (string): the URL to be tracked
                    [].sent (integer): the number of emails that contained the URL
                    [].clicks (integer): the number of times the URL has been clicked from a tracked email
-                   [].unique_clicks (integer): the number of unique emails that have benerated clicks for this URL
+                   [].unique_clicks (integer): the number of unique emails that have generated clicks for this URL
 
 
         Raises:
